from django.db import transaction
from common.rest_extend.response import Results, SUCCESS, get_error_status_code, RESTResponse
from common.rest_extend.views.related import RelatedView


class SynchronousError(Exception):
    pass


class Synchronous:
    def post(self, request, results, **kwargs):
        pass

    def put(self, request, results, **kwargs):
        pass

    def delete(self, request, results, **kwargs):
        pass

    def injection_post_data(self, data, **kwargs):
        pass

    def execute(self, request, results, data, **kwargs):
        return True


class CumulocityView(RelatedView):
    syn = Synchronous()

    def post(
            self,
            request,
            obj,
            obj_serializer,
            data=None,
            need_results=False,
            related_objs: list = None,
            m_to_m_objs: list = None,
            related_mapping=None,
            m_to_m_mapping=None,
            **kwargs,
    ):
        results = Results()
        try:
            with transaction.atomic():
                results = Results()
                if not data:
                    data = request.data
                if data.get("data"):
                    data = data.get("data")
                success = self.syn.execute(request, results, data)
                if not success:
                    return RESTResponse(results)
                serializer = obj_serializer(data=data)
                if not related_objs:
                    related_objs, related_mapping = self.injection_data.related_data("POST")
                if not m_to_m_objs:
                    m_to_m_objs, m_to_m_mapping = self.injection_data.m_to_m_data("POST")
                self.relate.execute_related_extend(related_objs, related_mapping, data)
                self.relate.execute_m_to_m_extend(m_to_m_objs, m_to_m_mapping, data)
                try:
                    valid = serializer.is_valid(raise_exception=True)
                    serializer.save()
                    results.describe = "add  successfully！！！"
                    results.code = 200
                except Exception as e:
                    results.code, results.describe = get_error_status_code(e)
                self.syn.post(request, results, data)
        except SynchronousError as e:
            pass
        except Exception as e:
            get_error_status_code(e, results)
        return RESTResponse(results)

    def put(
            self,
            request,
            obj,
            obj_serializer,
            data=None,
            pk="id",
            extend_conditions: dict = None,
            need_results=False,
            need_queryset=False,
            **kwargs,
    ):
        results = Results()
        try:
            if not data:
                data = request.data
            if data.get("data"):
                data = data.get("data")

            with transaction.atomic():
                results, queryset = super().put(
                    request,
                    obj,
                    obj_serializer,
                    data=data,
                    pk="externalid",
                    extend_conditions=extend_conditions,
                    need_results=need_results,
                    need_queryset=True,
                    raise_exception=True,
                    **kwargs,
                )
                if results.code < 300:
                    self.syn.put(request, results, data=data, queryset=queryset)
        except SynchronousError as e:
            pass
        except Exception as e:
            get_error_status_code(e, results)
        return RESTResponse(results)

    def delete(
            self,
            request,
            obj,
            obj_serializer,
            data=None,
            pk="id",
            extend_conditions: dict = None,
            need_results=False,
            need_queryset=False,
            **kwargs,
    ):
        results = Results()
        try:
            with transaction.atomic():
                results, instance = super().delete(
                    request,
                    obj=obj,
                    obj_serializer=obj_serializer,
                    data=data,
                    pk=pk,
                    extend_conditions=extend_conditions,
                    need_queryset=True,
                    **kwargs,
                )
                if instance:
                    self.syn.delete(request, results, instance=instance)
        except SynchronousError as e:
            pass
        except Exception as e:
            get_error_status_code(e, results)
        return RESTResponse(results)



from common.rest_extend.response import RESTResponse, Results, FORBID_CODE
from django.utils.deprecation import MiddlewareMixin
from tenant_auth.models import Tenant, TenantRole, Permission, RolePermissions

IGNORE_SOURCE = ["/"]


class AuthMiddleware(MiddlewareMixin):
    def process_request(self, request):
        path = request.path
        results = Results()
        if "login" not in path and path not in IGNORE_SOURCE:
            token = request.GET.get('token')
            if token:
                tenant = Tenant.objects.filter(token=token).first()
                authorization = self.permissions(request, tenant=tenant)
            else:
                account = request.session.get("account", None)
                if not account:
                    results.describe = "no permission!!!"
                    results.code = FORBID_CODE
                    return RESTResponse(results)
                authorization = self.permissions(request, account=account)
            if not authorization:
                results.describe = "no permission!!!"
                results.code = FORBID_CODE
                return RESTResponse(results)

    def permissions(self, request, account=None, tenant=None):

        method = request.method
        path = request.path
        if not tenant:
            tenant = Tenant.objects.filter(account=account).first()
        if tenant:
            account = tenant.account
            setattr(request, 'tenant', tenant)
            if tenant.account == "admin":
                return True
            # permission = AuthPermission.objects.filter(action=method, source_id=path).first()
            permission = Permission.objects.filter(source_id=path).first()
            if permission:
                tenant_role = TenantRole.objects.filter(tenant_id=account).first()
                if tenant_role:
                    role_permissions = RolePermissions.objects.filter(
                        role_id=tenant_role.role_id, permission_id=permission.name
                    ).first()
                    if role_permissions:
                        return True

        return False



from common.rest_extend.response import Results


class MySQLTransaction():
    def post(self, request, obj, obj_serializer, obj_relevance: list = None, m_to_m_obj: list = None,
             mapping=None,
             m_to_m_mapping=None, data=None):
        results = Results()
        if not data:
            data = request.data
        if data.get('data'):
            data = data.get('data')
        serializer = obj_serializer(data=data)
        if obj_relevance and mapping:
            for obj in obj_relevance:
                obj_name = obj.__name__.lower()
                obj_mapping = mapping.get(obj_name)
                if not obj_mapping:
                    raise Exception(f'{obj_name}  are not mapping!!!')
                queryset = self.get_relevance_queryset(obj, obj_name, data)
                self.extend_relevance_data(data, queryset, obj_mapping)
        if m_to_m_obj and m_to_m_mapping:
            for obj in m_to_m_obj:
                pass
        try:
            valid = serializer.is_valid(raise_exception=True)
            serializer.save()
            results.describe = "add  successfully！！！"
            results.status = SUCCESS
            results.code = 200
        except Exception as e:
            results.code, results.describe = get_error_status_code(e)

        return RESTResponse(results)



import json
from queue import Queue
from threading import Thread
from django.utils.deprecation import MiddlewareMixin
from middleware.estools import ESClient

from common.middlewares import logger


class EventMiddleware(MiddlewareMixin):

    def start(self, host, port, username=None, password=None, index='event'):
        self.q = Queue()
        self.index = index
        self.es_client = ESClient()
        self.es = self.es_client.conn(host=host, port=port, username=username, password=password)
        Thread(target=self.send_event).start()

    def process_response(self, request, response):
        try:
            if response.status_code < 300 and response.status_code >= 200 and request.method != 'GET':
                data = self.parsing(request, response)
                if data:
                    self.q.put(data)
        except Exception as e:
            logger.exception(e)
        return response

    def parsing(self, request, response):
        data = json.loads(request.body)
        return data

    def send_event(self):
        while True:
            try:
                body = self.q.get()
                self.es_client.index(index=self.index, body=body)

            except Exception as e:
                logger.exception(e)
